@@ -2,8 +2,8 @@ import torch
 import torch.nn as nn
 import torch.nn.functional as F
 import torch.linalg as LA
+from torch.amp import autocast
 from torch.nn.attention.flex_attention import flex_attention, create_block_mask
-if torch.cuda.is_available(): flex_attention = torch.compile(flex_attention)
 import math
 import warnings
 import opt_einsum
@@ -492,9 +492,8 @@ class SlidingWindowAttention(nn.Module):
         return mask
     
     @lru_cache(maxsize=2)
-    def causal_windowed_block_mask(self, src_len, tgt_len=None, device=None):
+    def causal_windowed_block_mask(self, src_len, tgt_len=None):
         if tgt_len is None: tgt_len = src_len
-        if device is None: device = self.device
         
         half_span = self.dilation * (self.window_len - 1)
         
@@ -502,12 +501,11 @@ class SlidingWindowAttention(nn.Module):
             diff = q_idx - kv_idx
             return (diff >= 0) & (diff <= half_span) & ((diff % self.dilation) == 0)
 
-        return create_block_mask(mask_mod, B=None, H=None, Q_LEN=src_len, KV_LEN=tgt_len, device=self.device)
+        return create_block_mask(mask_mod, B=None, H=None, Q_LEN=src_len, KV_LEN=tgt_len, BLOCK_SIZE=1, device=self.device)
     
     @lru_cache(maxsize=2)
-    def symmetric_windowed_block_mask(self, src_len, tgt_len=None, device=None):
+    def symmetric_windowed_block_mask(self, src_len, tgt_len=None):
         if tgt_len is None: tgt_len = src_len
-        if device is None: device = self.device
         
         half = self.window_len // 2
         half_span = self.dilation * half
@@ -516,7 +514,7 @@ class SlidingWindowAttention(nn.Module):
             diff = q_idx - kv_idx
             return diff.abs().le(half_span) & ((diff % self.dilation) == 0)
 
-        return create_block_mask(mask_mod,  B=None, H=None, Q_LEN=src_len, KV_LEN=tgt_len, device=device)
+        return create_block_mask(mask_mod,  B=None, H=None, Q_LEN=src_len, KV_LEN=tgt_len, BLOCK_SIZE=1, device=self.device)
     
     def forward(self, x, rope=None, causal=True):
         if self.batch_first:
@@ -545,7 +543,7 @@ class SlidingWindowAttention(nn.Module):
         
         beta = torch.exp(self.beta).reshape(self.n_heads, 1, 1).repeat(bsz, 1, 1)
         # beta = F.softplus(self.beta).reshape(self.n_heads, 1, 1).repeat(bsz, 1, 1)
-        q = q / (math.sqrt(self.d_head) * beta)
+        q = q / (math.sqrt(self.d_head) * beta).to(q.dtype)
         
         if not self.use_flex_attn:
             if causal:
@@ -574,27 +572,31 @@ class SlidingWindowAttention(nn.Module):
             
             attn_output = rearrange(attn_output, '(b h) s d -> s b (h d)', h=self.n_heads)
         else:
-            attn_device = "cuda" if self.device == "cuda" else "cpu"
-            
-            q = q.to(attn_device).to(torch.float32)
-            k = k.to(attn_device).to(torch.float32)
-            v = v.to(attn_device).to(torch.float32)
-            
-            if causal:
-                block_mask = self.causal_windowed_block_mask(src_len, tgt_len, device=attn_device)
-            else:
-                block_mask = self.symmetric_windowed_block_mask(src_len, tgt_len, device=attn_device)
-            
             if self.attn_sink:
                 k = torch.cat([k, torch.zeros((bsz*self.n_heads, 1, self.d_head), dtype=k.dtype, device=k.device)], dim=1)
                 v = torch.cat([v, torch.zeros((bsz*self.n_heads, 1, self.d_head), dtype=v.dtype, device=v.device)], dim=1)
                 tgt_len += 1
             
+            if causal:
+                block_mask = self.causal_windowed_block_mask(src_len, tgt_len)
+            else:
+                block_mask = self.symmetric_windowed_block_mask(src_len, tgt_len)
+            
             q = rearrange(q, '(b h) s d -> b h s d', h=self.n_heads).contiguous()
             k = rearrange(k, '(b h) s d -> b h s d', h=self.n_heads).contiguous()
             v = rearrange(v, '(b h) s d -> b h s d', h=self.n_heads).contiguous()
             
-            attn_output = flex_attention(q, k, v, block_mask=block_mask)[:, :, :src_len]
+            attn_device = "cuda" if self.device == "cuda" else "cpu"
+            
+            # autocast_dtype = torch.get_autocast_dtype(self.device)
+            
+            q = q.to(attn_device)#.to(autocast_dtype)
+            k = k.to(attn_device)#.to(autocast_dtype)
+            v = v.to(attn_device)#.to(autocast_dtype)
+
+            block_mask = block_mask.to(attn_device)#.to(autocast_dtype)
+
+            attn_output = flex_attention(q, k, v, block_mask=block_mask, scale=1.0)[:, :, :src_len]
             
             attn_output = rearrange(attn_output, 'b h s d -> s b (h d)', h=self.n_heads).contiguous().to(self.device)
         
