@@ -183,8 +183,11 @@ def train_epoch(epoch, train_loader, model, optimizer, loss_fn, log_fn=default_l
         
         if (batch_idx + 1) % accumulation_steps == 0:
             if grad_clip_norm is not None: torch.nn.utils.clip_grad_norm_(model.parameters(), max_norm=grad_clip_norm)
-            optimizer.step() if not mixed_precision else scaler.step(optimizer)
-            if mixed_precision: scaler.update()
+            if not mixed_precision:
+                optimizer.step()
+            else:
+                scaler.step(optimizer)
+                scaler.update()
             optimizer.zero_grad()
             if scheduler: scheduler.step()
             
@@ -224,8 +227,11 @@ def train_epoch(epoch, train_loader, model, optimizer, loss_fn, log_fn=default_l
     if (batch_idx + 1) % accumulation_steps != 0:
         completed_steps += 1
         if grad_clip_norm is not None: torch.nn.utils.clip_grad_norm_(model.parameters(), max_norm=grad_clip_norm)
-        optimizer.step() if not mixed_precision else scaler.step(optimizer)
-        if mixed_precision: scaler.update()
+        if not mixed_precision:
+            optimizer.step()
+        else:
+            scaler.step(optimizer)
+            scaler.update()
         optimizer.zero_grad()
         if scheduler: scheduler.step()
         
